<?php
namespace Pinpoint\Feed\Model;

//use Magento\Framework\Model\AbstractModel;

use Magento\Config\Model\Config\Reader\Source\Deployed\DocumentRoot;
use Magento\Framework\App\ObjectManager;
use Pinpoint\Feed\Helper\Data;

abstract class AbstractFeed //extends \Magento\Framework\DataObject
{
    const OPEN_TAG_KEY = 'start';

    const CLOSE_TAG_KEY = 'end';

    const INDEX_FILE_PREFIX = 'sitemap';

    const TYPE_INDEX = 'sitemap';

    const TYPE_URL = 'url';

    /**
     * Real file path
     *
     * @var string
     */
    protected $_filePath;

    /**
     * Sitemap start and end tags
     *
     * @var array
     */
    protected $_tags = [];

    /**
     * Current sitemap file size
     *
     * @var int
     */
    protected $_fileSize = 0;

    /**
     * New line possible symbols
     *
     * @var array
     */
    private $_crlf = ["win" => "\r\n", "unix" => "\n", "mac" => "\r"];

    /**
     * @var \Magento\Framework\Filesystem\Directory\Write
     */
    protected $_directory;

    /**
     * @var \Magento\Framework\Filesystem\File\Write
     */
    protected $_stream;

    /**
     * @var \Magento\Framework\Escaper
     */
    protected $_escaper;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    protected $_dateModel;

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $_request;

    /**
     * @var \Magento\Framework\Stdlib\DateTime
     */
    protected $dateTime;

    protected $io;

    protected $store;

    protected $helper;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        //\Magento\Framework\Registry $registry,
        \Magento\Framework\Escaper $escaper,
        \Magento\Framework\Filesystem $filesystem,
      //  \Magento\Sitemap\Model\ResourceModel\Catalog\ProductFactory $productFactory,
      //  \Magento\Sitemap\Model\ResourceModel\Cms\PageFactory $cmsFactory,
        Data $helper,
        \Magento\Framework\Stdlib\DateTime\DateTime $modelDate,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Framework\Stdlib\DateTime $dateTime,
        \Magento\Framework\Filesystem\Io\File $io,
      //  array $data = [],
        \Magento\Config\Model\Config\Reader\Source\Deployed\DocumentRoot $documentRoot = null
    ) {
        $this->_escaper = $escaper;
        $documentRoot = $documentRoot ?: ObjectManager::getInstance()->get(DocumentRoot::class);
        $this->_directory = $filesystem->getDirectoryWrite($documentRoot->getPath());
        //$this->_productFactory = $productFactory;
       // $this->_cmsFactory = $cmsFactory;
        $this->_dateModel = $modelDate;
        $this->helper = $helper;
       // $this->_storeManager = $storeManager;
        $this->_request = $request;
        $this->dateTime = $dateTime;
        $this->io = $io;

        //parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    public abstract function getFilename();

    /**
     * @return int
     */
    public function getStoreId()
    {
        return $this->store->getId();
    }

    public function setStore($store)
    {
        $this->store = $store;
    }

    public function getStore()
    {
        return $this->store;
    }

    public abstract function getFilePath();

    protected function _getUrl($url, $type = \Magento\Framework\UrlInterface::URL_TYPE_LINK)
    {
        return $this->_getStoreBaseUrl($type) . ltrim($url, '/');
    }

    protected function _getStoreBaseUrl($type = \Magento\Framework\UrlInterface::URL_TYPE_LINK)
    {
        /** @var \Magento\Store\Model\Store $store */
        $store = $this->store;
        $isSecure = $store->isUrlSecure();
        return rtrim($store->getBaseUrl($type, $isSecure), '/') . '/';
    }

    /**
     * Check sitemap file location and permissions
     *
     * @return \Magento\Framework\Model\AbstractModel
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function prepareFile()
    {
        $path = $this->getFilePath();

        /**
         * Check path is allow
         */
        if ($path && preg_match('#\.\.[\\\/]#', $path)) {
            throw new \Magento\Framework\Exception\LocalizedException(__('Please define a correct path: ' . $path));
        }
        /**
         * Check exists and writable path
         */
        if (!$this->_directory->isExist($path)) {
            $this->io->mkdir($path, 0775);
        }

        if (!$this->_directory->isWritable($path)) {
            throw new \Magento\Framework\Exception\LocalizedException(
                __('Please make sure that "%1" is writable by the web-server.', $this->getFilePath())
            );
        }
        /**
         * Check allow filename
         */
        if (!preg_match('#^[a-zA-Z0-9_\.]+$#', $this->getFilename())) {
            throw new \Magento\Framework\Exception\LocalizedException(
                __(
                    'Please use only letters (a-z or A-Z), numbers (0-9) or underscores (_) in the filename. No spaces or other characters are allowed: ' . $this->getFilename()
                )
            );
        }
    }

    protected function getFeedFilename()
    {
        if (!preg_match('#\.xml$#', $this->getFilename())) {
            return $this->getFilename() . '.xml';
        } else {
            return $this->getFilename();
        }
    }

    public function getFeedFilePath()
    {
        return rtrim(str_replace(str_replace('\\', '/', $this->_getBaseDir()), '', $this->getFilePath()), '/') . '/';
    }

    protected function _getBaseDir()
    {
        return $this->_directory->getAbsolutePath();
    }

    /**
     * Initialize sitemap
     *
     * @return void
     */
    protected function _initSitemapItems()
    {
        $lastModifiedDate = $this->dateTime->formatDate(true, true);

        $this->_tags = [
            'pages' => [
                self::OPEN_TAG_KEY => '<?xml version="1.0" encoding="UTF-8"?>' .
                    PHP_EOL .
                    '<data>' .
                    PHP_EOL,
                self::CLOSE_TAG_KEY => '</data>',
            ],
            self::TYPE_URL => [
                self::OPEN_TAG_KEY => '<?xml version="1.0" encoding="UTF-8"?>' .
                    PHP_EOL .
                    "<feed><lastModifiedDate>{$lastModifiedDate}</lastModifiedDate><data>" .
                    PHP_EOL,
                self::CLOSE_TAG_KEY => '</data></feed>',
            ],
        ];
    }

    protected function _createFeed($fileName = null, $type = self::TYPE_URL)
    {
        if (!$fileName) {
            $fileName = $this->getFeedFilename();
        }

        $path = rtrim($this->getFeedFilePath(), '/') . '/' . $fileName;
        $this->_stream = $this->_directory->openFile($path);

        $fileHeader = sprintf($this->_tags[$type][self::OPEN_TAG_KEY], $type);
        $this->_stream->write($fileHeader);
        $this->_fileSize = strlen($fileHeader . sprintf($this->_tags[$type][self::CLOSE_TAG_KEY], $type));
    }

    protected function _finalizeFile($type = self::TYPE_URL)
    {
        if ($this->_stream) {
            $this->_stream->write(sprintf($this->_tags[$type][self::CLOSE_TAG_KEY], $type));
            $this->_stream->close();
        }

        // Reset all counters
        $this->_lineCount = 0;
        $this->_fileSize = 0;
    }

    protected function _writeRow($row)
    {
        $this->_getStream()->write($row . PHP_EOL);
    }

    protected function _getStream()
    {
        if ($this->_stream) {
            return $this->_stream;
        } else {
            throw new \Magento\Framework\Exception\LocalizedException(__('File handler unreachable'));
        }
    }

    public abstract  function generateXml();

    protected abstract function _getRow($item);
}