<?php
namespace Pinpoint\Feed\Model;

use Pinpoint\Feed\Helper\Data;

class PageCmsFeed extends AbstractFeed
{
    /**
     * @var \Magento\Cms\Model\ResourceModel\Page\CollectionFactory
     */
    protected $pageFactory;

    protected $pageHelper;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        //\Magento\Framework\Registry $registry,
        \Magento\Framework\Escaper $escaper,
        \Magento\Framework\Filesystem $filesystem,
        Data $helper,
        //\Magento\Sitemap\Model\ResourceModel\Cms\PageFactory $cmsFactory,
        \Magento\Cms\Model\ResourceModel\Page\CollectionFactory $pageFactory,
        \Magento\Framework\Stdlib\DateTime\DateTime $modelDate,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Framework\Stdlib\DateTime $dateTime,
        \Magento\Framework\Filesystem\Io\File $io,
        \Magento\Cms\Helper\Page $pageHelper,
        //  array $data = [],
        \Magento\Config\Model\Config\Reader\Source\Deployed\DocumentRoot $documentRoot = null
    ) {
        $this->pageFactory = $pageFactory;
        $this->pageHelper = $pageHelper;
        parent::__construct($context, $escaper, $filesystem, $helper, $modelDate, $request, $dateTime, $io, $documentRoot);
    }

    public function generateXml()
    {
        if (!$this->helper->isPageFeedEnabled($this->getStoreId())) {
            return $this;
        }

        $this->prepareFile();
        $this->_lineCount = 0;

        $this->_initSitemapItems();

        /** @var $sitemapItem \Magento\Framework\DataObject */
        //$collection = $this->_productFactory->create()->addStoreFilter($storeId);
        $collection = $this->pageFactory->create()->addStoreFilter($this->getStoreId());

        foreach ($collection as $item) {
            $xml = $this->_getRow(
                $item
            );
//            if ($this->_isSplitRequired($xml) && $this->_sitemapIncrement > 0) {
//                $this->_finalizeSitemap();
            //          }
            if (!$this->_fileSize) {
                $this->_createFeed(null, 'pages');
            }
            $this->_writeRow($xml);
            // Increase counters
            $this->_lineCount++;
            $this->_fileSize += strlen($xml);
        }

        $this->_finalizeFile('pages');

        return $this;
    }

    public function getFilename()
    {
        return $this->helper->getPageFeedFilename($this->getStoreId());
    }

    public function getFilePath()
    {
        return $this->helper->getPageFeedFilepath($this->getStoreId());
    }

    /**
     * @param  \Magento\Cms\Model\Page $item
     * @return string
     */
    protected function _getRow($item)
    {
        $row = '<id>' . $item->getId() . '</id>';
        $row .= '<name><![CDATA[' . $item->getTitle() . ']]></name>';
        $row .= '<desc><![CDATA[' . $item->getContent() . ']]></desc>';
        $row .= '<metaDesc><![CDATA[' . $item->getMetaDescription() . ']]></metaDesc>';
        $row .= '<metaKeywords><![CDATA[' . $item->getMetaKeywords() . ']]></metaKeywords>';
        $row .= '<category>page</category>';
        $row .= '<url>' . htmlspecialchars($this->pageHelper->getPageUrl($item->getId())) . '</url>';

        return '<record>' . $row . '</record>';
    }
}